/* 
 * CDDL HEADER START
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License 
 * (the License). You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the License at
 * http://www.sun.com/cddl/cddl.html and legal/CDDLv1.0.txt
 * See the License for the specific language governing
 * permission and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file  
 * at legal/CDDLv1.0.txt.                                                           
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * your own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems Inc. All Rights Reserved
 * CDDL HEADER END
*/

package com.sun.faces.portlet;


import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.faces.lifecycle.Lifecycle;
import javax.faces.lifecycle.LifecycleFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>Custom implementation of <code>LifecycleFactory</code> that
 * provides the portlet-specific <code>Lifecycle</code> by default.</p>
 */

public final class LifecycleFactoryImpl extends LifecycleFactory {

    // The Log instance for this class
    private static final Log log = LogFactory.getLog(LifecycleFactoryImpl.class);
    
    // ------------------------------------------------------------ Constructors


    public LifecycleFactoryImpl() {
        if (log.isTraceEnabled()) {
	    log.trace("Created LifecycleFactory " + this);
	}
        addLifecycle(LifecycleFactory.DEFAULT_LIFECYCLE, new LifecycleImpl());
    }


    // ------------------------------------------------------ Instance Variables


    // Registered Lifecycle instances, keyed by lifecycle identifier
    private Map lifecycles = new HashMap();


    // ---------------------------------------------------------- Public Methods


    public void addLifecycle(String lifecycleId, Lifecycle lifecycle) {

        if ((lifecycleId == null) || (lifecycle == null)) {
            throw new NullPointerException();
        }
        synchronized (lifecycles) {
            if (lifecycles.containsKey(lifecycleId)) {
                throw new IllegalArgumentException(lifecycleId);
            }
            lifecycles.put(lifecycleId, lifecycle);
        }
        if (log.isTraceEnabled()) {
	    log.trace("Added LifecycleId " + lifecycleId);
	}

    }


    public Lifecycle getLifecycle(String lifecycleId) {

        if (lifecycleId == null) {
            throw new NullPointerException();
        }
        synchronized (lifecycles) {
            Lifecycle lifecycle = (Lifecycle) lifecycles.get(lifecycleId);
            if (lifecycle != null) {
                if (log.isTraceEnabled()) {
	            log.trace("Returned " + lifecycle + " for lifecycleId "
                            + lifecycleId);
	        }
                return (lifecycle);
            } else {
                throw new IllegalArgumentException(lifecycleId);
            }
        }

    }


    public Iterator getLifecycleIds() {

        synchronized (lifecycles) {
            return (lifecycles.keySet().iterator());
        }

    }


}
